    //
    //  RecursiveKeyValueObserving.h
    //  RecursiveKeyValueObserving
    //
    //  Created by Pascal Werz on 06/08/12.
    //  Copyright (c) 2012 Pascal Werz. All rights reserved.
    //

#import <Foundation/Foundation.h>


    // This framework uses ObjC Automatic Reference Counting


    // This protocol is required for objects to support recursive observing
@protocol RecursiveKeyValueObservingProtocol

@required
- (NSArray *) recursiveObservedKeyNames;	// returns array of key names to observe

@end



    // This class implement recursive observing
@interface RecursiveKeyValueObservingManager : NSObject
{
@private
    NSMutableSet * _recursiveKeyValueObservings;
    void * _reference;
    __unsafe_unretained NSObject <RecursiveKeyValueObservingProtocol> * _managedObject;
}

@property (strong, readwrite) NSMutableSet * recursiveKeyValueObservings;
@property (assign, readwrite) void * reference;
@property (assign, readwrite) NSObject <RecursiveKeyValueObservingProtocol> * managedObject;

+ (RecursiveKeyValueObservingManager *) managerForObject:(NSObject <RecursiveKeyValueObservingProtocol> *)object managingId:(void *)managingId;

- (void) addRecursiveObserver:(id) observer
                      options:(NSKeyValueObservingOptions) options
                      context:(void *) context;

- (void) removeRecursiveObserver:(id) observer
                         context:(void *) context;

- (void) copyObserversFromObject:(NSObject <RecursiveKeyValueObservingProtocol> *) object;

- (void) copyObserversFromObject:(NSObject <RecursiveKeyValueObservingProtocol> *) object forKey:(NSString *) key;

- (void) removeAllRecursiveObserversForKey:(NSString *) key;

- (void) removeAllRecursiveObservers;

@end